module LifenFhir
  class Channel < Element

    attribute :type, String
    attribute :value, String

    attribute :telecom, LifenFhir::Telecom

    def fhir_payload(user)
      raise Error.new(message: "Invalid channel: an UUID is required") if !valid?

      {
        id: user.uuid,
        resourceType: "Practitioner",
        type => [
          {
            id: uuid
          }
        ]
      }
    end

    def valid?
      uuid and uuid.length != 0
    end

    def self.from_json(json, type = nil)
      if type == "telecom"
        telecom = Telecom.new.attributes_from_json(json)

        new(uuid: telecom.uuid, type: type, value: telecom.value)
      else

        new(uuid: json["id"], type: type, value: "#{Array(json["line"]).join(", ")}, #{json["postalCode"]} #{json["city"]}")
      end
    end

  end
end
